function [B,A,H]=xcloe(y,r,na,nb,nc,d,R,S,T,Fin,lam1,lam0)
% XCLOE	is used to identify a discrete time model of a plant operating in 
%		closed-loop with an RST controller based on the X-CLOE method
%
%		[B,A,H]=xcloe(y,r,na,nb,nh,d,R,S,T,Fin,lam1,lam0)
%               
%		y and r are the column vectors containing respectively the output and the 
%		excitation signal.
%
%		na, nb are the order of the polynomials A,B and d is the pure time delay.
%		nh is the order of the numerator of the noise model in closed loop
%		( na<=nh<= max(na+ns,nb+d+nr)).
% 
%		R, S and T are the column vectors containing the parameters of a two
%		degree of freedom controller. S*u(t)=-R*y(t)+T*r(t)
% 		Remark: when the excitation signal is added to the measured output 
%		(i.e. the controller is in feedforward with unit feedback) we have T=R and  
%		when the excitation signal is added to the control input (i.e. the controller
%		is in feedback) we have T=S.	
%
%		Fin is the initial gain F0=Fin*(na+nb+nh)*eye(na+nb+nh) (Fin=1000 by default)
%
% 		lam1 and lam0 make different adaptation algoritms as follows:
%		
%		lam1=1;lam0=1   	 :decreasing gain (default algorithm)
%		0.95<lam1<1;lam0=1 :decreasing gain with fixed forgetting factor 
%		0.95<lam1,lam0<1	 :decreasing gain with variable forgetting factor 
%
%		See also cloe, fcloe, afcloe and clvalid.
%		Copyright by Adaptech, 1997-1999


[nl,ncy]=size(y);
if ncy>2, error('This routine is only for SISO systems'),end
[nl,ncr]=size(r);
if ncr>2, error('This routine is only for SISO systems'),end
if (na<0 || nb<0 || nc<0 || d<0), error('The order of A,B,C and d should not be negative!!'),end

R=R./S(1);T=T./S(1);S=S./S(1);
[rs,cs]=size(S);
[rr,cr]=size(R);
[rt,ct]=size(T);
if min(size(R))>1, error('R should be a column vector!');end
if min(size(S))>1, error('S should be a column vector!');end
if min(size(T))>1, error('T should be a column vector!');end
if cs>rs, S=S';end
if cr>rr, R=R';end
if ct>rt, T=T';end

nd=min(length(r),length(y));	% number of data
nr=length(R);
ns=length(S);
nth=na+nb+nc;

if nd<nth, error('Number of data should be greater than the number of parameters!'),end	

np=max([na,nb+d,ns,nc])+1;

if nargin<9, error('This routin needs more parameters!'),end 
if nargin<10, lam1=1;lam0=1;Fin=1000;end
if nargin<11, lam1=1;lam0=1;end
if nargin<12, lam0=1;end

if isempty(Fin), Fin=1000;end
if isempty(lam1),lam1=1;end
if isempty(lam0), lam0=1;end

if (lam1>1 || lam0>1), error('lam1 and lam0 should be less than 1');end
if (lam1<0.95 || lam0<0.95), disp ('warning :lam1 and lam0 are normally greater than 0.95');end

theta=zeros(nth,1);
phi=zeros(3*np,1);

nT=length(T);
rt=[zeros(nT+1,1)];

F=Fin*nth*eye(nth);

i=[1:np-1 np+1:2*np-1 2*np+1:3*np-1];		%shift index for vector phi
j=[1:na np+d+1:np+d+nb 2*np+1:2*np+nc];  		%phi index for theta


for t=1:nd
	yhat=theta'*phi(j);
	e_apri=y(t)-yhat;
	e_apost=e_apri/(1+phi(j)'*F*phi(j));
	theta=theta+F*phi(j)*e_apost;

	F=1/lam1*(F-((F*phi(j)*phi(j)'*F)/(lam1+phi(j)'*F*phi(j))));
	lam1=lam0*lam1+1-lam0;

	yhat=theta'*phi(j);

	%Compute uhat by the control law
      rt(2:nT+1)=rt(1:nT);rt(1)=r(t);
	if ns>1, Su=S(2:ns)'*phi(np+1:np+ns-1);else Su=0;end
      uhat=R'*[-yhat;phi(1:nr-1)]-Su+T'*rt(1:nT);

 
 %Filter e_apost by 1/S
 %sep=S(2:ns)'*phi(2*np+1:2*np+ns-1);
 %if isempty(sep), sep=0;
 
 x=S(2:ns);
 if isempty(x), sep =0;
 else  
 sep =x'*phi(2*np+1:2*np+ns-1);
 end
 e_ap_f=e_apost-sep;
	phi(i+1)=phi(i);phi(1)=-yhat;phi(np+1)=uhat;phi(2*np+1)=e_ap_f;

end
A=[1;theta(1:na)]';
B=[zeros(d+1,1);theta(na+1:na+nb)]';
H=[1;theta(na+nb+1:nth)]';